@@ -4,6 +4,7 @@ Common solar physics coordinate systems.
 This submodule implements various solar physics coordinate frames for use with
 the `astropy.coordinates` module.
 """
+import warnings
 from contextlib import contextmanager
 
 import numpy as np
@@ -155,6 +156,35 @@ class SunPyBaseCoordinateFrame(BaseCoordinateFrame):
         else:
             return super().__str__()
 
+    @property
+    def _is_2d(self):
+        return (self._data is not None and self._data.norm().unit is u.one
+                and u.allclose(self._data.norm(), 1*u.one))
+
+    def __init_subclass__(cls, **kwargs):
+        super().__init_subclass__(**kwargs)
+
+        # TODO: Remove this after the minimum Astropy dependency includes astropy/astropy#12005
+        cls._fix_property_docstrings()
+
+    @classmethod
+    def _fix_property_docstrings(cls):
+        # This class method adds docstrings to properties dynamically created by
+        # BaseCoordinateFrame.__init_subclass__().  Accordingly, this method needs to itself be
+        # called from SunPyBaseCoordinateFrame.__init_subclass__() to work for our subclasses.
+        property_docstrings = {
+            'default_representation': "Default representation for position data",
+            'default_differential': "Default representation for differential data",
+            'frame_specific_representation_info': "Mapping for frame-specific component names",
+        }
+        for prop, docstring in property_docstrings.items():
+            if getattr(cls, prop).__doc__ is None:
+                setattr(getattr(cls, prop), '__doc__', docstring)
+
+
+# TODO: Remove this after the minimum Astropy dependency includes astropy/astropy#12005
+SunPyBaseCoordinateFrame._fix_property_docstrings()
+
 
 class BaseHeliographic(SunPyBaseCoordinateFrame):
     """
@@ -191,9 +221,7 @@ class BaseHeliographic(SunPyBaseCoordinateFrame):
         frame : `~sunpy.coordinates.frames.BaseHeliographic`
             The fully 3D coordinate
         """
-        # Check if the coordinate is 2D
-        if (self._data is not None and self._data.norm().unit is u.one
-                and u.allclose(self._data.norm(), 1*u.one)):
+        if self._is_2d:
             return self.realize_frame(self._data * self.rsun)
 
         # The coordinate is already 3D
@@ -268,8 +296,8 @@ class HeliographicCarrington(BaseHeliographic):
 
     This system differs from Stonyhurst Heliographic (HGS) in its definition of longitude.  This
     longitude is an "apparent" longitude because it takes into account the time it takes for light
-    to travel from the Sun's surface to the observer.  Thus, the observer needs to be specified to
-    be able to transform to any other coordinate frame.
+    to travel from the Sun's surface to the observer (see :ref:`sunpy-coordinates-carrington`).
+    Thus, the observer needs to be specified to be able to transform to any other coordinate frame.
 
     A new instance can be created using the following signatures
     (note that if supplied, ``obstime`` and ``observer`` must be a keyword argument)::
@@ -325,6 +353,12 @@ class HeliographicCarrington(BaseHeliographic):
 
     observer = ObserverCoordinateAttribute(HeliographicStonyhurst)
 
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        if not isinstance(self.observer, BaseCoordinateFrame) and self.observer == 'self' and self._is_2d:
+            raise ValueError("Full 3D coordinate (including radius) must be specified "
+                             "when observer='self'.")
+
 
 @add_common_docstring(**_frame_parameters())
 class Heliocentric(SunPyBaseCoordinateFrame):
@@ -484,8 +518,7 @@ class Helioprojective(SunPyBaseCoordinateFrame):
             now with a third coordinate.
         """
         # Skip if we already are 3D
-        distance = self.spherical.distance
-        if not (distance.unit is u.one and u.allclose(distance, 1*u.one)):
+        if not self._is_2d:
             return self
 
         if not isinstance(self.observer, BaseCoordinateFrame):
@@ -498,11 +531,12 @@ class Helioprojective(SunPyBaseCoordinateFrame):
 
         # Check for the use of floats with lower precision than the native Python float
         if not set([lon.dtype.type, lat.dtype.type]).issubset([float, np.float64, np.longdouble]):
-            raise SunpyUserWarning("The Helioprojective component values appear to be lower "
-                                   "precision than the native Python float: "
-                                   f"Tx is {lon.dtype.name}, and Ty is {lat.dtype.name}. "
-                                   "To minimize precision loss, you may want to cast the values to "
-                                   "`float` or `numpy.float64` via the NumPy method `.astype()`.")
+            warnings.warn("The Helioprojective component values appear to be lower "
+                          "precision than the native Python float: "
+                          f"Tx is {lon.dtype.name}, and Ty is {lat.dtype.name}. "
+                          "To minimize precision loss, you may want to cast the values to "
+                          "`float` or `numpy.float64` via the NumPy method `.astype()`.",
+                          SunpyUserWarning)
 
         # Calculate the distance to the surface of the Sun using the law of cosines
         cos_alpha = np.cos(lat) * np.cos(lon)
